Package: GVS
Title: 'Geocoordinate Validation Service'
Version: 0.0.2
Authors@R: c(
    person("Brian", "Maitner", email = "bmaitner@gmail.com",
    role = c("aut", "cre"),
    comment = c(ORCID = "https://orcid.org/0000-0002-2118-9880")),
    person("Brad", "Boyle", email = "bboyle@email.arizona.edu", role = c("aut")),
    person("Rethvick Sriram Yugendra", "Babu", email = "rethvick@arizona.edu",
    role = c("aut"))
    )
Description: The 'Geocoordinate Validation Service' (GVS) runs checks of coordinates in latitude/longitude format. It returns annotated coordinates with additional flags and metadata that can be used in data cleaning.  Additionally, the package has functions related to attribution and metadata information. More information can be found at <https://github.com/ojalaquellueva/gvs/tree/master/api>.
Depends: R (>= 3.5.0)
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
LazyData: true
Imports: jsonlite, httr
Suggests: knitr, rmarkdown, testthat, devtools, BIEN, vcr (>= 0.6.0)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-29 18:28:59 UTC; bmaitner
Author: Brian Maitner [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2118-9880>),
  Brad Boyle [aut],
  Rethvick Sriram Yugendra Babu [aut]
Maintainer: Brian Maitner <bmaitner@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-29 18:40:17 UTC
Built: R 4.6.0; ; 2026-05-01 21:15:40 UTC; unix

# Generated by roxygen2: do not edit by hand

export(GVS)
export(GVS_citations)
export(GVS_collaborators)
export(GVS_data_dictionary)
export(GVS_metadata)
export(GVS_sources)
export(GVS_version)
import(httr)
importFrom(httr,GET)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)

#'Get information on sources used by the GVS
#'
#'Return metadata about the current GVS sources
#' @param ... Additional arguments passed to internal functions.
#' @return Dataframe containing information about the sources used in the current GVS version.
#' @export
#' @examples {
#' sources <- GVS_sources()
#' }
#'
GVS_sources <- function(...){

  # Check for internet access
  if (!check_internet()) {
    message("This function requires internet access, please check your connection.")
    return(invisible(NULL))
  }

  return(gvs_core(mode = "sources",...))

}#NSR sources

context("gvs sources")


test_that("example works", {
  # skip_if_offline(host = "r-project.org")


  vcr::use_cassette("gvs_sources", {
    sources <- GVS_sources(
      url = url,
      skip_internet_check = TRUE
    )
  })


  # test below assume a dataframe and will be skipped if one isn't returned
  skip_if_not(class(sources) == "data.frame")
  expect_gt(object = nrow(sources), expected = 0)
})

#'Get collaborator information
#'
#'Returns information on GVS collaborators
#' @param ... Additional arguments passed to internal functions.
#' @return Dataframe containing bibtex-formatted citation information
#' @export
#' @examples {
#' collaborator_info <- GVS_collaborators()
#' }
#'
GVS_collaborators <- function(...){

  return(gvs_core(mode="collaborators",...))

}#NSR citations

#' Check whether the internet is on
#'
#' Check for internet
#' @return TRUE if internet connection is detected, FALSE otherwise.
#' @importFrom httr GET
#' @keywords internal
check_internet <- function() {
  tryCatch(class(httr::GET("http://www.google.com/")) == "response",
    error = function(e) {
      return(FALSE)
    }
  )
}

#'Check the validity of coordinates
#'
#'GVS returns information on coordinate validity.
#' @param occurrence_dataframe A properly formatted dataframe, see `gvs_testfile`
#' @param ... Additional arguments passed to internal functions.
#' @return Dataframe containing GVS results.
#' @export
#' @examples \donttest{
#'
#'results <- GVS(occurrence_dataframe = gvs_testfile)
#'
#' }
GVS <- function(occurrence_dataframe,
                ...){

  #check that input is a data.frame

    if(!inherits(occurrence_dataframe,"data.frame")){
      stop("occurrence_dataframe should be a data.frame")
    }

  #check for two columns

  if(ncol(occurrence_dataframe)!=2){

    warning("Occurrence dataframe should have exactly two columns: one for latitude and one for longitude (in that order).")
    return(invisible(NULL))

  }


  # Convert the data to JSON
  data_json <- toJSON(unname(occurrence_dataframe))

  results <- gvs_core(mode = "resolve",
                       data_json = data_json, ...)

  # results <- gvs_core(mode = "resolve",
  #                     data_json = data_json)
  #

  #If the API didn't work, return a null
  if(is.null(results)){
    return(invisible(NULL))
  }

  #Re-order results to match original data
  # results <- results[match(table = results$user_id,
  #                          x = occurrence_dataframe$user_id),]

  #reset the row numbers
  rownames(results) <- NULL

  return(results)

}

context("gvs main")


test_that("example works", {
  # skip_if_offline(host = "r-project.org")

  vcr::use_cassette("gvs_example", {

    results <- GVS(occurrence_dataframe = gvs_testfile,
                   url=url,
                   skip_internet_check = TRUE)

  })


  # test below assume a data dictionary and will be skipped if one isn't returned
  skip_if_not(class(results) == "data.frame")
  expect_equal(object = nrow(results), expected = nrow(gvs_testfile))
})

#'Get data dictionary
#'
#'Returns the GVS data dictionary
#' @param ... Additional arguments passed to internal functions.
#' @return Dataframe containing bibtex-formatted citation information
#' @export
#' @examples {
#' data_dictionary <- GVS_data_dictionary()
#' }
#'
GVS_data_dictionary <- function(...){

  return(gvs_core(mode="dd",...))

}#NSR citations

context("gvs data dictionary")


test_that("example works", {
  # skip_if_offline(host = "r-project.org")


  vcr::use_cassette("gvs_dd", {

    GVS_fields <- GVS_data_dictionary(url = url,
                                      skip_internet_check = TRUE)
  })


  # test below assume a data.frame and will be skipped if one isn't returned
  skip_if_not(class(GVS_fields) == "data.frame")
  expect_gt(object = nrow(GVS_fields), expected = 2)
})

#'Get citation information
#'
#'Returns information needed to cite the GVS
#' @param ... Additional arguments passed to internal functions.
#' @return Dataframe containing bibtex-formatted citation information
#' @note This function provides citation information in bibtex format that can be used with reference manager software (e.g. Paperpile, Zotero). Please do remember to cite both the sources and the GVS, as the GVS couldn't exist without these sources!
#' @export
#' @examples {
#' citation_info <- GVS_citations()
#' }
#'
GVS_citations <- function(...){

  return(gvs_core(mode="citations",...))

}#GVS citations

context("gvs citations")


test_that("example works", {
  # skip_if_offline(host = "r-project.org")


  vcr::use_cassette("gvs_citations", {
    citation_info <- GVS_citations(
      url = url,
      skip_internet_check = TRUE
    )
  })


  # test below assume a data.frame and will be skipped if one isn't returned
  skip_if_not(class(citation_info) == "data.frame")
  expect_gt(object = nrow(citation_info), expected = 1)

})

# api urls

# URL for NSR API
url <- "https://gvsapi.xyz/gvs_api.php" # production
#url <- "http://vegbiendev.nceas.ucsb.edu:7775/gvs_api.php" #testing

# Bad URLs for testing

# url <- "www.google.com"
# url <- "www.hisstank.com"

library("vcr") # *Required* as vcr is set up on loading

invisible(vcr::vcr_configure(
  dir = vcr::vcr_test_path("fixtures")
))

#vcr::check_cassette_names()

#' Example GVS data
#'
#' A sample dataset showing the proper formatting of GVS inputs.
#'
#' @format A data.frame with 27 observations of 2 variables:
#' \describe{
#'   \item{Latitude}{Latitude, in decimal degrees}
#'   \item{Longitude}{Longitude, in decimal degrees}
#'   ...
#' }
#' @source \url{https://biendata.org}
"gvs_testfile"

#'Get GVS metadata
#'
#'Returns metadata on GVS including version and citation information
#' @param bibtex_file Optional output file for writing bibtex citations.
#' @param ... Additional arguments passed to internal functions.
#' @return List containing: (1) bibtex-formatted citation information, (2) information about GVS data sources, and (3) GVS version information.
#' @note This function provides citation information in bibtex format that can be used with reference manager software (e.g., Paperpile, Zotero). Please remember to cite both the sources and the GVS, as the GVS couldn't exist without these sources!
#' @note This function is a wrapper that returns the output of the functions GVS_citations, GVS_sources, and GVS_version.
#' @export
#' @examples {
#' metadata <- GVS_metadata()
#' }
#'
GVS_metadata <- function(bibtex_file=NULL, ...){

  output <- list()

  output[[1]] <- GVS_citations(...)
  output[[2]] <- GVS_sources(...)
  output[[3]] <- GVS_version(...)
  
  if(length(output)< 3){
    
    message("One or more of the component API calls failed.")
    return(invisible(NULL))
    
  }

  names(output) <- c("citations","version","sources")

  #Write the bibtex information if a file is specified
  
  if(!is.null(bibtex_file)){
    writeLines(text = output$citations$citation,
               con = bibtex_file)}

  return(output)

}

context("gvs metadata")


test_that("example works", {
  # because of the way this call is structured, I can't capture it via vcr, so I skip it on cran/offline
  skip_if_offline(host = "r-project.org")
  skip_on_cran()

  metadata <- GVS_metadata()

  # test below assume a data dictionary and will be skipped if one isn't returned

  skip_if_not(class(metadata) == "list")

  expect_equal(object = class(metadata[[1]]), expected = "data.frame")
  expect_equal(object = class(metadata[[2]]), expected = "data.frame")
  expect_equal(object = class(metadata[[3]]), expected = "data.frame")

  expect_equal(object = length(metadata), expected = 3)
})

#'Handle API access and format
#'
#'Internal function for handling accessing the API,data formatting, and errors
#' @param url Server URL to use.  Defaults to the stable production version
#' @param mode API mode to use.
#' @param data_json Either NULL (the default) or properly formatted json
#' @param batches NULL or Numeric.  Optional number of batches to divide the request into for parallel processing.
#' @importFrom jsonlite toJSON fromJSON
#' @import httr
#' @keywords internal
gvs_core <- function(url = "https://gvsapi.xyz/gvs_api.php" ,
                     mode,
                     data_json = NULL,
                     batches = NULL,
                     skip_internet_check = FALSE){

  # Check for internet access
  if (!skip_internet_check) {
    if (!check_internet()) {
      message("This function requires internet access, please check your connection.")
      return(invisible(NULL))
    }
  }

  # Construct the request
  headers <- list('Accept' = 'application/json', 'Content-Type' = 'application/json', 'charset' = 'UTF-8')

  # Set the opts
  opts <- data.frame(c(mode))
  names(opts) <- c("mode")

  #Format the input
  if(is.null(data_json)){

    opts_json <- jsonlite::toJSON(opts)
    opts_json <- gsub('\\[','',opts_json)
    opts_json <- gsub('\\]','',opts_json)
    input_json <- paste0('{"opts":', opts_json, '}' )


  }else{

    if ( exists("batches") ){opts$batches <- batches}

    opts_json <-  jsonlite::toJSON(opts)
    opts_json <- gsub('\\[','',opts_json)
    opts_json <- gsub('\\]','',opts_json)

    # Combine the options and data into single JSON object
    input_json <- paste0('{"opts":', opts_json, ',"data":', data_json, '}' )

  }


  # Send the request in a "graceful failure" wrapper for CRAN compliance
  tryCatch(expr = results_json <- POST(url = url,
                                       add_headers('Content-Type' = 'application/json'),
                                       add_headers('Accept' = 'application/json'),
                                       add_headers('charset' = 'UTF-8'),
                                       body = input_json,
                                       encode = "json"
  )
  ,
  error = function(e) {
    message("There appears to be a problem reaching the API.")
  })


  #Return NULL if API isn't working

  if(!exists("results_json")){
    return(invisible(NULL))
  }

  #Check status, if it doesn't equal

  if(results_json$status_code != 200){

    message(paste("Problem with the API: HTTP Status" ,results_json$status_code))
    #fromJSON(rawToChar(results_json$content))

    return(invisible(NULL))

  }

  # Ensure that the results are properly formatted and return results or a message.

  tryCatch(expr = results_raw <- fromJSON(rawToChar(results_json$content)),
           error = function(e) {
             message(paste("There seems to be a problem with the query, which returned the following: \n",rawToChar(results_json$content)))
           })

  #Convert to data.frame if things worked

  if(!exists("results_raw")){
    return(invisible(NULL))
  }else{
    results <- as.data.frame(results_raw)
  }


  return(results)

}

#'Get metadata on current GVS version
#'
#'Return metadata about the current GVS version
#' @param ... Additional arguments passed to internal functions.
#' @return Dataframe containing current GVS version number, build date, and code version.
#' @export
#' @examples{
#' NSR_version_metadata <- GVS_version()
#' }
#'
GVS_version <- function(...){

  # Check for internet access
  if (!check_internet()) {
    message("This function requires internet access, please check your connection.")
    return(invisible(NULL))
  }

  return(gvs_core(mode = "meta",...))

}#NSR version

context("gvs version")


test_that("example works", {
  # skip_if_offline(host = "r-project.org")

  vcr::use_cassette("gvs_version", {
    GVS_version_metadata <- GVS_version(
      url = url,
      skip_internet_check = TRUE
    )
  })


  # test below assume a data dictionary and will be skipped if one isn't returned
  skip_if_not(class(GVS_version_metadata) == "data.frame")
  expect_equal(object = nrow(GVS_version_metadata), expected = 1)
})

---
title: "GVS R package"
author: "Brian Maitner"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{GVS R package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Geocoordinate Validation Service

The package `GVS` is designed to interact with the Geocoordinate Validation Service API of the Botanical Information and Ecology Network. The GVS accepts a point of observation (PO; pair of latitude, longitude coordinates in decimal degrees) and returns the country, state and county in which the point is located. It also calculates the distance between the OP and six different types of centroids for each of the three political divisions (see Centroid Types) and indicates for each political division if it is small enough for the OP itself to potentially be a centroid (see Distance Thresholds. Finally, the GVS indicates which of the three potential political division centroids is the most likely, if any, based on the threshold parameter MAX_DIST (distance to the actual centroid) and MAX_DIST_REL (relative distance: distance to actual centroid / distance from actual centroid to the farthest point in the political division). In addition, the GVS validates the submitted coordinates and reports errors such as non-numeric values and values out of bounds. Valid coordinates which do not join to any political division are flagged as "Point in ocean".


## Installation
```{r, eval=FALSE}

library(devtools)
install_github("EnquistLab/RGVS")


```

## Using the GVS

The GVS takes as input a dataframe consisting of two columns: latitude and longitude (in that order). We provide an example dataset, `gvs_testfile`.

```{r}

library(GVS)

# Load the sample dataset

gvs_data <- gvs_testfile

# View the structure
head(gvs_data)

```

To run data through the GVS, we use the function `GVS`

```{r}

gvs_results <- GVS(occurrence_dataframe = gvs_data)

# The resulting output has a lot of information that can be used in data cleaning.

colnames(gvs_results)


```
The contents of some of these columns may not be immediately clear. In this case, we can consult the data dictionary.

```{r}

dd <- GVS_data_dictionary()


```

We can now use these metadata to exclude questionable data (or try to fix it).  For example, we'll certainly want to exclude coordinates that are non-numeric or impossible!  Depending on our purposes, we may also want to exclude coordinates that correspond to centroids or those that fall into the ocean.

## Additional information about the GVS

In addition to the coordinate resolution, the GVS offers metadata responses describing current code and database versions, and formatted citations. For a quick overview, you can use the function `GVS_metadata`. If you use the GVS in a publication and need to cite it, proper attribution information is available with `GVS_citations`.

```{r}

cites <- GVS_citations()

# The citation column provides Bibtex format citations that can be pasten into a reference manager (e.g., EndNote, PaperPile, Zotero)

cites$citation

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_internet.R
\name{check_internet}
\alias{check_internet}
\title{Check whether the internet is on}
\usage{
check_internet()
}
\value{
TRUE if internet connection is detected, FALSE otherwise.
}
\description{
Check for internet
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gvs_core.R
\name{gvs_core}
\alias{gvs_core}
\title{Handle API access and format}
\usage{
gvs_core(
  url = "https://gvsapi.xyz/gvs_api.php",
  mode,
  data_json = NULL,
  batches = NULL,
  skip_internet_check = FALSE
)
}
\arguments{
\item{url}{Server URL to use.  Defaults to the stable production version}

\item{mode}{API mode to use.}

\item{data_json}{Either NULL (the default) or properly formatted json}

\item{batches}{NULL or Numeric.  Optional number of batches to divide the request into for parallel processing.}
}
\description{
Internal function for handling accessing the API,data formatting, and errors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GVS.R
\name{GVS}
\alias{GVS}
\title{Check the validity of coordinates}
\usage{
GVS(occurrence_dataframe, ...)
}
\arguments{
\item{occurrence_dataframe}{A properly formatted dataframe, see \code{gvs_testfile}}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing GVS results.
}
\description{
GVS returns information on coordinate validity.
}
\examples{
\donttest{

results <- GVS(occurrence_dataframe = gvs_testfile)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GVS_sources.R
\name{GVS_sources}
\alias{GVS_sources}
\title{Get information on sources used by the GVS}
\usage{
GVS_sources(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing information about the sources used in the current GVS version.
}
\description{
Return metadata about the current GVS sources
}
\examples{
{
sources <- GVS_sources()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GVS_citations.R
\name{GVS_citations}
\alias{GVS_citations}
\title{Get citation information}
\usage{
GVS_citations(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing bibtex-formatted citation information
}
\description{
Returns information needed to cite the GVS
}
\note{
This function provides citation information in bibtex format that can be used with reference manager software (e.g. Paperpile, Zotero). Please do remember to cite both the sources and the GVS, as the GVS couldn't exist without these sources!
}
\examples{
{
citation_info <- GVS_citations()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GVS_metadata.R
\name{GVS_metadata}
\alias{GVS_metadata}
\title{Get GVS metadata}
\usage{
GVS_metadata(bibtex_file = NULL, ...)
}
\arguments{
\item{bibtex_file}{Optional output file for writing bibtex citations.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
List containing: (1) bibtex-formatted citation information, (2) information about GVS data sources, and (3) GVS version information.
}
\description{
Returns metadata on GVS including version and citation information
}
\note{
This function provides citation information in bibtex format that can be used with reference manager software (e.g., Paperpile, Zotero). Please remember to cite both the sources and the GVS, as the GVS couldn't exist without these sources!

This function is a wrapper that returns the output of the functions GVS_citations, GVS_sources, and GVS_version.
}
\examples{
{
metadata <- GVS_metadata()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gvs_testfile}
\alias{gvs_testfile}
\title{Example GVS data}
\format{
A data.frame with 27 observations of 2 variables:
\describe{
\item{Latitude}{Latitude, in decimal degrees}
\item{Longitude}{Longitude, in decimal degrees}
...
}
}
\source{
\url{https://biendata.org}
}
\usage{
gvs_testfile
}
\description{
A sample dataset showing the proper formatting of GVS inputs.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GVS_data_dictionary.R
\name{GVS_data_dictionary}
\alias{GVS_data_dictionary}
\title{Get data dictionary}
\usage{
GVS_data_dictionary(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing bibtex-formatted citation information
}
\description{
Returns the GVS data dictionary
}
\examples{
{
data_dictionary <- GVS_data_dictionary()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GVS_version.R
\name{GVS_version}
\alias{GVS_version}
\title{Get metadata on current GVS version}
\usage{
GVS_version(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing current GVS version number, build date, and code version.
}
\description{
Return metadata about the current GVS version
}
\examples{
{
NSR_version_metadata <- GVS_version()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GVS_collaborators.R
\name{GVS_collaborators}
\alias{GVS_collaborators}
\title{Get collaborator information}
\usage{
GVS_collaborators(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing bibtex-formatted citation information
}
\description{
Returns information on GVS collaborators
}
\examples{
{
collaborator_info <- GVS_collaborators()
}

}
